IncompleteOnConflict

Struct IncompleteOnConflict 

pub struct IncompleteOnConflict<Stmt, Target> { /* private fields */ }
A partially constructed ON CONFLICT clause.

Implementations

Source

impl<T: QuerySource, U, Op, Ret, Target> IncompleteOnConflict<InsertStatement<T, U, Op, Ret>, Target>

pub fn do_nothing(self) -> DoNothing<Self>

Creates a query with ON CONFLICT (target) DO NOTHING

If you want to do nothing when any constraint conflicts, use on_conflict_do_nothing instead. See on_conflict for usage examples.

pub fn do_update(self) -> DoUpdate<Self>

Used to create a query in the form ON CONFLICT (...) DO UPDATE ... [WHERE ...]

Call .set on the result of this function with the changes you want to apply. The argument to set can be anything that implements AsChangeset (e.g. anything you could pass to set on a normal update statement).

Note: When inserting more than one row at a time, this query can still fail if the rows being inserted conflict with each other.

For some backends (PostgreSQL, SQLite) a WHERE clause can be used to limit the rows actually updated. For PostgreSQL and SQLite you can use the .filter() method to add conditions like that.

Examples
Set specific value on conflict

PostgreSQL/SQLite:

let user = User {
    id: 1,
    name: "Pascal",
};
let user2 = User {
    id: 1,
    name: "Sean",
};

assert_eq!(
    Ok(1),
    diesel::insert_into(users).values(&user).execute(conn)
);

let insert_count = diesel::insert_into(users)
    .values(&user2)
    .on_conflict(id)
    .do_update()
    .set(name.eq("I DONT KNOW ANYMORE"))
    .execute(conn);
assert_eq!(Ok(1), insert_count);
assert_eq!(Ok(2), insert_count);

let users_in_db = users.load(conn);
assert_eq!(
    Ok(vec![(1, "I DONT KNOW ANYMORE".to_string())]),
    users_in_db
);

MySQL:

let user = User {
    id: 1,
    name: "Pascal",
};
let user2 = User {
    id: 1,
    name: "Sean",
};

assert_eq!(
    Ok(1),
    diesel::insert_into(users).values(&user).execute(conn)
);

diesel::insert_into(users)
    .values(&user2)
    .on_conflict(diesel::dsl::DuplicatedKeys)
    .do_update()
    .set(name.eq("I DONT KNOW ANYMORE"))
    .execute(conn)?;

let users_in_db = users.load(conn);
assert_eq!(
    Ok(vec![(1, "I DONT KNOW ANYMORE".to_string())]),
    users_in_db
);
Set AsChangeset struct on conflict

PostgreSQL & SQLite:

let user = User {
    id: 1,
    name: "Pascal",
};
let user2 = User {
    id: 1,
    name: "Sean",
};

assert_eq!(
    Ok(1),
    diesel::insert_into(users).values(&user).execute(conn)
);

let insert_count = diesel::insert_into(users)
    .values(&user2)
    .on_conflict(id)
    .do_update()
    .set(&user2)
    .execute(conn);
assert_eq!(Ok(1), insert_count);

let users_in_db = users.load(conn);
assert_eq!(Ok(vec![(1, "Sean".to_string())]), users_in_db);

MySQL:


let user = User {
    id: 1,
    name: "Pascal",
};
let user2 = User {
    id: 1,
    name: "Sean",
};

assert_eq!(
    Ok(1),
    diesel::insert_into(users).values(&user).execute(conn)
);

diesel::insert_into(users)
    .values(&user2)
    .on_conflict(diesel::dsl::DuplicatedKeys)
    .do_update()
    .set(&user2)
    .execute(conn)?;

let users_in_db = users.load(conn);
assert_eq!(Ok(vec![(1, "Sean".to_string())]), users_in_db);
Use excluded to get the rejected value
use diesel::upsert::excluded;

let user = User {
    id: 1,
    name: "Pascal",
};
let user2 = User {
    id: 1,
    name: "Sean",
};
let user3 = User {
    id: 2,
    name: "Tess",
};

assert_eq!(
    Ok(1),
    diesel::insert_into(users).values(&user).execute(conn)
);

#[cfg(feature = "postgres")]
let insert_count = diesel::insert_into(users)
    .values(&vec![user2, user3])
    .on_conflict(id)
    .do_update()
    .set(name.eq(excluded(name)))
    .execute(conn);
assert_eq!(Ok(2), insert_count);

let users_in_db = users.load(conn);
assert_eq!(
    Ok(vec![(1, "Sean".to_string()), (2, "Tess".to_string())]),
    users_in_db
);
Use .filter()method to limit the rows actually updated
use self::users::dsl::*;
let user = User {
    id: 1,
    name: "Pascal",
};
let user2 = User {
    id: 1,
    name: "Sean",
};

assert_eq!(
    Ok(1),
    diesel::insert_into(users).values(&user).execute(conn)
);

let insert_count = diesel::insert_into(users)
    .values(&user2)
    .on_conflict(id)
    .do_update()
    .set(&user2)
    .filter(id.ge(5))
    .execute(conn);
assert_eq!(Ok(0), insert_count);

let users_in_db = users.load(conn);
assert_eq!(Ok(vec![(1, "Pascal".to_string())]), users_in_db);